@@ -28,7 +28,7 @@ class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
     stringify_titles(data)
     sort_unordered_metaparams(data)
     munge_edges(data)
-    synthesize_edges(data)
+    synthesize_edges(data, catalog)
 
     hash
   end
@@ -72,25 +72,25 @@ class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
       # is both an optimization and a safeguard.
       next if real_resource.key_attributes.count > 1
 
+      aliases = [real_resource[:alias]].flatten.compact
+
       # Non-isomorphic resources aren't unique based on namevar, so we can't
       # use it as an alias
       type = real_resource.resource_type
-      next if type.respond_to?(:isomorphic?) and !type.isomorphic?
+      if !type.respond_to?(:isomorphic?) or type.isomorphic?
+        # This makes me a little sad.  It turns out that the "to_hash" method
+        #  of Puppet::Resource can have side effects.  In particular, if the
+        #  resource type specifies a title_pattern, calling "to_hash" will trigger
+        #  the title_pattern processing, which can have the side effect of
+        #  populating the namevar (potentially with a munged value).  Thus,
+        #  it is important that we search for namevar aliases in that hash
+        #  rather than in the resource itself.
+        real_resource_hash = real_resource.to_hash
 
-      aliases = [real_resource[:alias]].flatten.compact
-
-      # This makes me a little sad.  It turns out that the "to_hash" method
-      #  of Puppet::Resource can have side effects.  In particular, if the
-      #  resource type specifies a title_pattern, calling "to_hash" will trigger
-      #  the title_pattern processing, which can have the side effect of
-      #  populating the namevar (potentially with a munged value).  Thus,
-      #  it is important that we search for namevar aliases in that hash
-      #  rather than in the resource itself.
-      real_resource_hash = real_resource.to_hash
-
-      name = real_resource_hash[real_resource.send(:namevar)]
-      unless name.nil? or real_resource.title == name or aliases.include?(name)
-        aliases << name
+        name = real_resource_hash[real_resource.send(:namevar)]
+        unless name.nil? or real_resource.title == name or aliases.include?(name)
+          aliases << name
+        end
       end
 
       resource['parameters']['alias'] = aliases unless aliases.empty?
@@ -141,11 +141,25 @@ class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
     resources.find {|res| res['type'] == resource_hash['type'] and res['title'].to_s == resource_hash['title'].to_s}
   end
 
-  def synthesize_edges(hash)
+  def synthesize_edges(hash, catalog)
     aliases = map_aliases_to_title(hash)
 
     hash['resources'].each do |resource|
-      next if resource['exported']
+      # Standard virtual resources don't appear in the catalog. However,
+      # exported resources which haven't been also collected will appears as
+      # exported and virtual (collected ones will only be exported). They will
+      # eventually be removed from the catalog, so we can't add edges involving
+      # them. Puppet::Resource#to_pson_data_hash omits 'virtual', so we have to
+      # look it up in the catalog to find that information. This isn't done in
+      # a separate step because we don't actually want to send the field (it
+      # will always be false). See ticket #16472.
+      #
+      # The outer conditional is here because Class[main] can't properly be
+      # looked up using catalog.resource and will return nil. See ticket
+      # #16473. Yay.
+      if real_resource = catalog.resource(resource['type'], resource['title'])
+        next if real_resource.virtual?
+      end
 
       Relationships.each do |param,relation|
         if value = resource['parameters'][param]
@@ -154,16 +168,39 @@ class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
 
             resource_hash = {'type' => resource['type'], 'title' => resource['title']}
             other_hash = resource_ref_to_hash(other_ref)
+
+            # This is an unfortunate hack.  Puppet does some weird things w/rt
+            # munging off trailing slashes from file resources, and users may
+            # legally specify relationships using a different number of trailing
+            # slashes than the resource was originally declared with.
+            # We do know that for any file resource in the catalog, there should
+            # be a canonical entry for it that contains no trailing slashes.  So,
+            # here, in case someone has specified a relationship to a file resource
+            # and has used one or more trailing slashes when specifying the
+            # relationship, we will munge off the trailing slashes before
+            # we look up the resource in the catalog to create the edge.
+            if other_hash['type'] == 'File' and other_hash['title'] =~ /\/$/
+              other_hash['title'] = other_hash['title'].sub(/\/+$/, '')
+            end
+
             other_array = [other_hash['type'], other_hash['title']]
 
             # Try to find the resource by type/title or look it up as an alias
             # and try that
             other_resource = find_resource(hash['resources'], other_hash) || find_resource(hash['resources'], aliases[other_array])
 
-            raise "Can't synthesize edge: #{resource_hash_to_ref(resource_hash)} -#{relation[:relationship]}- #{other_ref} (param #{param})" unless other_resource
+            raise "Invalid relationship: #{edge_to_s(resource_hash_to_ref(resource_hash), other_ref, param)}, because #{other_ref} doesn't seem to be in the catalog" unless other_resource
 
-            if other_resource['exported']
-              raise "Can't create an edge between #{resource_hash_to_ref(resource_hash)} and exported resource #{other_ref}"
+            # As above, virtual exported resources will eventually be removed,
+            # so if a real resource refers to one, it's wrong. Non-virtual
+            # exported resources are exported resources that were also
+            # collected in this catalog, so they're okay. Virtual non-exported
+            # resources can't appear in the catalog in the first place, so it
+            # suffices to check for virtual.
+            if other_real_resource = catalog.resource(other_resource['type'], other_resource['title'])
+              if other_real_resource.virtual?
+                raise "Invalid relationship: #{edge_to_s(resource_hash_to_ref(resource_hash), other_ref, param)}, because #{other_ref} is exported but not collected"
+              end
             end
 
             # If the ref was an alias, it will have a different title, so use
@@ -194,4 +231,8 @@ class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
   def resource_hash_to_ref(hash)
     "#{hash['type']}[#{hash['title']}]"
   end
+
+  def edge_to_s(specifier_resource, referred_resource, param)
+    "#{specifier_resource} { #{param} => #{referred_resource} }"
+  end
 end
