@@ -4,4 +4,5 @@ $LOAD_PATH.unshift File.join(dir, "../lib")
 # don't fail any worse than we already would.
 $LOAD_PATH.push File.join(dir, "../../../puppetlabs_spec_helper")
 
-require 'puppet_spec_helper'
+require 'rspec'
+require 'puppetlabs_spec_helper/puppet_spec_helper'
