@@ -163,6 +163,13 @@
         "CREATE INDEX idx_catalog_resources_tags_gin ON catalog_resources USING gin(tags)")
       (log/warn (format "Version %s of PostgreSQL is too old to support fast tag searches; skipping GIN index on tags. For reliability and performance reasons, consider upgrading to the latest stable version." (string/join "." (sql-current-connection-database-version)))))))
 
+(defn drop-old-tags-index
+  "Drops the non-GIN tags index, which is never used (because nobody
+  does equality comparisons against array columns)"
+  []
+  (sql/do-commands
+   "DROP INDEX idx_catalog_resources_tags"))
+
 ;; The available migrations, as a map from migration version to migration
 ;; function.
 (def migrations
@@ -170,7 +177,10 @@
    2 allow-node-deactivation
    3 add-catalog-timestamps
    4 add-certname-facts-metadata-table
-   5 add-missing-indexes})
+   5 add-missing-indexes
+   6 drop-old-tags-index})
+
+(def desired-schema-version (apply max (keys migrations)))
 
 (defn schema-version
   "Returns the current version of the schema, or 0 if the schema
@@ -208,6 +218,12 @@ along with the time at which the migration was performed."
   "Migrates database to the latest schema version. Does nothing if database is
 already at the latest schema version."
   []
+  (let [current-schema-version (schema-version)]
+    (when (< desired-schema-version current-schema-version)
+      (throw (IllegalStateException.
+              (format "This version of PuppetDB works with schema version %d, but the DB is at version %d"
+                      desired-schema-version current-schema-version)))))
+
   (if-let [pending (seq (pending-migrations))]
     (sql/transaction
      (doseq [[version migration] pending]
